import json
import os
import re
import subprocess


def parse_subprocess_result(result):
    stdout = result.stdout
    stderr = result.stderr
    try:
        retval_bytes = re.search(b"DH-OUTPUT:(.+)\n", stdout).group(1)
    except AttributeError:
        error = stderr.decode("utf-8")
        raise RuntimeError(
            f"{error}\n\n Could not collect any result from the run_function in the main process because an error happened in the subprocess."
        )
    # Finally, parse whether the return value from the user-defined function is a scalar, a list, or a dictionary.
    retval = retval_bytes.replace(
        b"'", b'"'
    )  # For dictionaries, replace single quotes with double quotes!
    sol = json.loads(retval)
    return sol

def run_mpi(config, dequed, ranks_per_task, ranks_per_node):
    host = ",".join(dequed)
    cobalt_nodefile = os.environ["COBALT_NODEFILE"]
    runner = f"mpirun -x LD_LIBRARY_PATH -x PATH -x PYTHONPATH  -n {ranks_per_task} -N {ranks_per_node} --host {host} --hostfile {cobalt_nodefile} python script_horovod.py".split()

    # print(f"{runner=}")

    result = subprocess.run(runner, capture_output=True)

    res = parse_subprocess_result(result)

    return res

if __name__ == "__main__":
    ranks_per_task = 2
    ranks_per_node = 2
    run_mpi(config={}, dequed=[f"localhost:{ranks_per_node}"], ranks_per_task=ranks_per_task, ranks_per_node=ranks_per_node)

