import functools
import os
import subprocess

from deephyper.evaluator.callback import TqdmCallback, SearchEarlyStopping
from deephyper.problem import HpProblem
from deephyper.search.hps import CBO
from deephyper.evaluator import ProcessPoolEvaluator, queued
from run import run_mpi

def get_thetagpu_nodelist():
    nodefile = os.environ["COBALT_NODEFILE"]
    with open(nodefile) as nodefile:
        lines = nodefile.readlines()
        nodelist = [line.rstrip() for line in lines]
    return nodelist

if __name__ == "__main__":
    #Add HyperParameters
    problem = HpProblem()
    #problem.add_hyperparameter((8, 256, "log-uniform"), "batch_size", default_value=32)
    problem.add_hyperparameter((1e-4, 0.01, "log-uniform"), "learning_rate", default_value=0.001)
    #problem.add_hyperparameter((10, 100), "num_epochs", default_value=50)

    # resources per task
    ranks_per_task = 2
    ranks_per_node = 2
    nodes_per_task = ranks_per_task // ranks_per_node

    # resources in total 
    max_ranks_per_node = 2
    nodes = get_thetagpu_nodelist()
    max_ranks_total = len(nodes) * max_ranks_per_node
    num_workers = max_ranks_total // ranks_per_task

    n_jobs_per_node = max_ranks_per_node // ranks_per_node
    host = [f"{n}:{ranks_per_node}" for n in nodes] * n_jobs_per_node

    run_function = functools.partial(run_mpi, ranks_per_task=ranks_per_task, ranks_per_node=ranks_per_node)

    evaluator = queued(ProcessPoolEvaluator)(run_function, num_workers=num_workers, queue=host, queue_pop_per_task=nodes_per_task, callbacks=[TqdmCallback(), SearchEarlyStopping()])
    print("Evaluator queue: ", evaluator.queue)
    print("Evaluator queue_pop: ", evaluator.queue_pop_per_task)

    search = CBO(problem, evaluator)

    results = search.search(max_evals=5)
